//
//	ShapeView.swift
//
//	Created by LayoutCode
//

import UIKit

/*
 A wrapper view for CAShapeLayer so the layer can support auto layout.
 */
class ShapeView: UIView {
	
	override class var layerClass: AnyClass {
		return CAShapeLayer.self
	}
	
	var bezierPath: UIBezierPath!{
		set{
			theLayer.path = newValue?.cgPath
			if let gradientLayer = gradientLayer(){
				if gradientLayer.mask == nil{
					let mask = CAShapeLayer()
					gradientLayer.mask = mask
				}
				(gradientLayer.mask as? CAShapeLayer)?.path = theLayer.path
			}
			
		}
		get{
			if let cgPath = theLayer.path{
				return UIBezierPath(cgPath: cgPath)
			}
			return nil
		}
	}
	
	var theLayer: CAShapeLayer{
		get{
			return self.layer as! CAShapeLayer
		}
	}
	
	override var bounds: CGRect{
		set{
			super.bounds = newValue
			//Update bezier path if view bounds updated
			updateBezierPath()
			gradientLayer()?.frame = bounds
		}
		get{
			return super.bounds
		}
	}
	
	//Handle animation when bounds changed
	override func action(for layer: CALayer, forKey event: String) -> CAAction? {
		if event == "path"
		{
			if let action = super.action(for: layer, forKey: "backgroundColor") as? CAPropertyAnimation
			{
				let animation = CABasicAnimation(keyPath: event)
				animation.fromValue = theLayer.path
				animation.autoreverses = action.autoreverses
				animation.beginTime = action.beginTime
				animation.delegate = action.delegate
				animation.duration = action.duration
				animation.fillMode = action.fillMode
				animation.repeatCount = action.repeatCount
				animation.repeatDuration = action.repeatDuration
				animation.speed = action.speed
				animation.timingFunction = action.timingFunction
				animation.timeOffset = action.timeOffset
				return animation
			}
		}
		return super.action(for: layer, forKey: event)
	}
	
	func updateBezierPath(){
		if let path = bezierPath{
			let bounds = path.bounds
			let newBounds = self.bounds
			var scaleX: CGFloat = 1
			var scaleY: CGFloat = 1
			if bounds.width > 0{
				scaleX = newBounds.width/bounds.width
			}
			if bounds.height > 0{
				scaleY = newBounds.height/bounds.height
			}
			let scale = CGAffineTransform(scaleX: scaleX, y: scaleY)
			path.apply(scale)
			self.bezierPath = path
			(gradientLayer()?.mask as? CAShapeLayer)?.path = theLayer.path
		}
	}
}
