//
//	UIView+Gradient.swift
//
//	Created by LayoutCode
//

import Foundation
import UIKit

struct Gradient{
	var colors: [UIColor]
	var locations: [CGFloat]! = nil
	var startPoint: CGPoint
	var endPoint: CGPoint
}

extension UIView{
	
	public func gradientLayer(createIfNeeded: Bool = false) -> CAGradientLayer!{
		var gradientLayer: CAGradientLayer!
		if let sublayers = layer.sublayers{
			for sublayer in sublayers{
				if let name = sublayer.name, name == "_gradient_"{
					gradientLayer = sublayer as? CAGradientLayer
					break
				}
			}
		}
		
		//Create new gradient layer if not found
		if gradientLayer == nil, createIfNeeded{
			gradientLayer = CAGradientLayer()
			gradientLayer.name = "_gradient_"
			layer.insertSublayer(gradientLayer!, at: 0)
			gradientLayer.frame = bounds
		}
		return gradientLayer
	}
	
	var gradient: Gradient!{
		set{
			if let gradient = newValue{
				let gradientLayer = gradientLayer(createIfNeeded: true)
				gradientLayer?.startPoint = gradient.startPoint
				gradientLayer?.endPoint = gradient.endPoint
				gradientLayer?.colors = gradient.colors.map({$0.cgColor})
				gradientLayer?.locations = gradient.locations?.map({NSNumber(value: $0)})
			}else
			{
				gradientLayer()?.removeFromSuperlayer()
			}
		}
		get{
			if let gradientLayer = gradientLayer(){
				var colors = [UIColor]()
				if let cgColors = gradientLayer.colors{
					for color in cgColors{
						colors.append(UIColor(cgColor: color as! CGColor))
					}
				}
				let locations = gradientLayer.locations as? [CGFloat]
				let gradient = Gradient(colors: colors, locations: locations, startPoint: gradientLayer.startPoint, endPoint: gradientLayer.endPoint)
				return gradient
			}
			return nil
		}
	}
	
}

