//
//	SmartHomeDatasource.swift
//
//	Code generated by LayoutCode
//	www.layoutcodeapp.com
//

import UIKit

class SmartHomeDatasource: NSObject {
	
	enum SectionType{
		case main
		case camera
		case favourite
		case room
	}
	
	struct MainStatusModel: Hashable {
		let accessoryImage: UIImage
		let typeText: String
		let valueText: String
		let identifier = UUID()
		func hash(into hasher: inout Hasher) {
			hasher.combine(identifier)
		}
	}
	struct CameraModel: Hashable {
		let cameraImage: UIImage
		let lastUpdatedDuration: String
		let hideNoResponse: Bool
		let identifier = UUID()
		func hash(into hasher: inout Hasher) {
			hasher.combine(identifier)
		}
	}
	struct FavouriteDeviceModel: Hashable {
		let roomText: String
		let accessoryName: String
		let statusText: String
		let accessoryImage: UIImage
		let identifier = UUID()
		func hash(into hasher: inout Hasher) {
			hasher.combine(identifier)
		}
	}
	
	enum Item: Hashable {
		case mainStatus(MainStatusModel)
		case camera(CameraModel)
		case favouriteDevice(FavouriteDeviceModel)
	}
	
	struct Section: Hashable {
		let type: SectionType
		let items: [Item]
		
		func hash(into hasher: inout Hasher) {
			hasher.combine(type)
		}
	}
	
	var sections = [Section]()
	
	override init() {
		super.init()
		setupInitialData()
	}
	
	// MARK: Initial data
	
	///These are initial data set from LayoutCode. You can use these for static data, or replace them with a new data using provided helper function.
	func setupInitialData(){
		let section0Item00 = Item.mainStatus(MainStatusModel(accessoryImage: UIImage(named: "mainStatusView-imageView_0Image.jpg")!,typeText: "Climate",valueText: "50-109"))
		let section0Item10 = Item.mainStatus(MainStatusModel(accessoryImage: UIImage(named: "mainStatusView-imageView_1Image.jpg")!,typeText: "Lights",valueText: "6  On"))
		let section0Item20 = Item.mainStatus(MainStatusModel(accessoryImage: UIImage(named: "mainStatusView-imageView_2Image.jpg")!,typeText: "Security",valueText: "3 Locked"))
		let section00 = Section(type: .main, items: [section0Item00, section0Item10, section0Item20])
		
		let section1Item01 = Item.camera(CameraModel(cameraImage: UIImage(named: "cameraView-placeholder_0Image.jpg")!,lastUpdatedDuration: "10s",hideNoResponse: true))
		let section1Item11 = Item.camera(CameraModel(cameraImage: UIImage(named: "cameraView-placeholder_1Image.jpg")!,lastUpdatedDuration: "10s",hideNoResponse: false))
		let section1Item21 = Item.camera(CameraModel(cameraImage: UIImage(named: "cameraView-placeholder_2Image.jpg")!,lastUpdatedDuration: "13s",hideNoResponse: true))
		let section1Item31 = Item.camera(CameraModel(cameraImage: UIImage(named: "cameraView-placeholderImage.jpg")!,lastUpdatedDuration: "10s",hideNoResponse: false))
		let section1Item41 = Item.camera(CameraModel(cameraImage: UIImage(named: "cameraView-placeholderImage.jpg")!,lastUpdatedDuration: "1s",hideNoResponse: true))
		let section1Item51 = Item.camera(CameraModel(cameraImage: UIImage(named: "cameraView-placeholderImage.jpg")!,lastUpdatedDuration: "10s",hideNoResponse: true))
		let section11 = Section(type: .camera, items: [section1Item01, section1Item11, section1Item21, section1Item31, section1Item41, section1Item51])
		
		let section2Item02 = Item.favouriteDevice(FavouriteDeviceModel(roomText: "Living Room",accessoryName: "TV",statusText: "Off",accessoryImage: UIImage(named: "favouriteDeviceView-accessory_0Image.jpg")!))
		let section2Item12 = Item.favouriteDevice(FavouriteDeviceModel(roomText: "Living Room",accessoryName: "Left Light",statusText: "Off",accessoryImage: UIImage(named: "favouriteDeviceView-accessory_1Image.jpg")!))
		let section2Item22 = Item.favouriteDevice(FavouriteDeviceModel(roomText: "Hallway",accessoryName: "Hallway Light",statusText: "Off",accessoryImage: UIImage(named: "favouriteDeviceView-accessory_2Image.jpg")!))
		let section2Item32 = Item.favouriteDevice(FavouriteDeviceModel(roomText: "Living Room",accessoryName: "Right Light",statusText: "Off",accessoryImage: UIImage(named: "favouriteDeviceView-accessory_3Image.jpg")!))
		let section2Item42 = Item.favouriteDevice(FavouriteDeviceModel(roomText: "Bedroom",accessoryName: "Main Light",statusText: "On",accessoryImage: UIImage(named: "favouriteDeviceView-accessory_4Image.jpg")!))
		let section22 = Section(type: .favourite, items: [section2Item02, section2Item12, section2Item22, section2Item32, section2Item42])
		
		let section3Item03 = Item.favouriteDevice(FavouriteDeviceModel(roomText: "Living Room",accessoryName: "Left Light",statusText: "Off",accessoryImage: UIImage(named: "favouriteDeviceView-accessory_0Image.jpg")!))
		let section3Item13 = Item.favouriteDevice(FavouriteDeviceModel(roomText: "Living Room",accessoryName: "Fan",statusText: "Off",accessoryImage: UIImage(named: "favouriteDeviceView-accessory_1Image.jpg")!))
		let section33 = Section(type: .room, items: [section3Item03, section3Item13])
		
		sections = [section00, section11, section22, section33]
	}
	
	// MARK: Helper
	
	func getSection(type: SectionType) -> Section!{
		for section in sections {
			if section.type == type{
				return section
			}
		}
		return nil
	}
	
	func replaceSection(_ section: Section){
		let idx = sections.firstIndex { aSection in
			if aSection.type == section.type{
				return true
			}
			return false
		}
		if let idx = idx{
			sections.remove(at: idx)
			sections.insert(section, at: idx)
		}
	}
	
	///Replace all items at specified section. Use this function to replace the section data.
	func replaceItemModels(sectionType: SectionType, models: [Any]) {
		var theItems = [Item]()
		for model in models{
			if let model = model as? MainStatusModel{
				let item = Item.mainStatus(model)
				theItems.append(item)
			}
			if let model = model as? CameraModel{
				let item = Item.camera(model)
				theItems.append(item)
			}
			if let model = model as? FavouriteDeviceModel{
				let item = Item.favouriteDevice(model)
				theItems.append(item)
			}
		}
		let newSection = Section(type: sectionType, items: theItems)
		replaceSection(newSection)
	}
	
	///Insert model items at specified section using  start index
	func insertItemModels(sectionType: SectionType, models: [Any], startIndex: Int) {
		var theItems = [Item]()
		for model in models{
			if let model = model as? MainStatusModel{
				let item = Item.mainStatus(model)
				theItems.append(item)
			}
			if let model = model as? CameraModel{
				let item = Item.camera(model)
				theItems.append(item)
			}
			if let model = model as? FavouriteDeviceModel{
				let item = Item.favouriteDevice(model)
				theItems.append(item)
			}
		}
		if let section = getSection(type: sectionType){
			var items = section.items
			items.insert(contentsOf: theItems, at: startIndex)
			let newSection = Section(type: sectionType, items: items)
			replaceSection(newSection)
		}
	}
	    
	///Remove model items at specified section using given range
	func removeItemModels(sectionType: SectionType, range: Range<Int>) {
		if let section = getSection(type: sectionType){
			var items = section.items
			items.removeSubrange(range)
			let newSection = Section(type: sectionType, items: items)
			replaceSection(newSection)
		}
	}
	
}
